@@ -31,7 +31,7 @@ enum bln_values {
   BLN_CURBUF = 1,   // May re-use curbuf for new buffer
   BLN_LISTED = 2,   // Put new buffer in buffer list
   BLN_DUMMY  = 4,   // Allocating dummy buffer
-  // TODO(mhinz): merge patch that introduces BLN_NEW
+  BLN_NEW    = 8,   // create a new buffer
   BLN_NOOPT  = 16,  // Don't copy options to existing buffer
 };
 
@@ -63,35 +63,6 @@ enum bfa_values {
 # include "buffer.h.generated.h"
 #endif
 
-// Find a window that contains "buf" and switch to it.
-// If there is no such window, use the current window and change "curbuf".
-// Caller must initialize save_curbuf to NULL.
-// restore_win_for_buf() MUST be called later!
-static inline void switch_to_win_for_buf(buf_T *buf,
-                                         win_T **save_curwinp,
-                                         tabpage_T **save_curtabp,
-                                         bufref_T *save_curbuf)
-{
-  win_T *wp;
-  tabpage_T *tp;
-
-  if (!find_win_for_buf(buf, &wp, &tp)
-      || switch_win(save_curwinp, save_curtabp, wp, tp, true) == FAIL) {
-    switch_buffer(save_curbuf, buf);
-  }
-}
-
-static inline void restore_win_for_buf(win_T *save_curwin,
-                                       tabpage_T *save_curtab,
-                                       bufref_T *save_curbuf)
-{
-  if (save_curbuf->br_buf == NULL) {
-    restore_win(save_curwin, save_curtab, true);
-  } else {
-    restore_buffer(save_curbuf);
-  }
-}
-
 static inline void buf_set_changedtick(buf_T *const buf,
                                        const varnumber_T changedtick)
   REAL_FATTR_NONNULL_ALL REAL_FATTR_ALWAYS_INLINE;
@@ -103,6 +74,8 @@ static inline void buf_set_changedtick(buf_T *const buf,
 static inline void buf_set_changedtick(buf_T *const buf,
                                        const varnumber_T changedtick)
 {
+  typval_T old_val = buf->changedtick_di.di_tv;
+
 #ifndef NDEBUG
   dictitem_T *const changedtick_di = tv_dict_find(
       buf->b_vars, S_LEN("changedtick"));
@@ -116,6 +89,13 @@ static inline void buf_set_changedtick(buf_T *const buf,
   assert(changedtick_di == (dictitem_T *)&buf->changedtick_di);
 #endif
   buf->changedtick_di.di_tv.vval.v_number = changedtick;
+
+  if (tv_dict_is_watched(buf->b_vars)) {
+    tv_dict_watcher_notify(buf->b_vars,
+                           (char *)buf->changedtick_di.di_key,
+                           &buf->changedtick_di.di_tv,
+                           &old_val);
+  }
 }
 
 static inline varnumber_T buf_get_changedtick(const buf_T *const buf)
@@ -145,15 +125,4 @@ static inline void buf_inc_changedtick(buf_T *const buf)
   buf_set_changedtick(buf, buf_get_changedtick(buf) + 1);
 }
 
-#define WITH_BUFFER(b, code) \
-  do { \
-    win_T *save_curwin = NULL; \
-    tabpage_T *save_curtab = NULL; \
-    bufref_T save_curbuf = { NULL, 0, 0 }; \
-    switch_to_win_for_buf(b, &save_curwin, &save_curtab, &save_curbuf); \
-    code; \
-    restore_win_for_buf(save_curwin, save_curtab, &save_curbuf); \
-  } while (0)
-
-
 #endif  // NVIM_BUFFER_H
