@@ -59,7 +59,9 @@
 #define BUFUNL        0x20000   /* accepts unlisted buffer too */
 #define ARGOPT        0x40000   /* allow "++opt=val" argument */
 #define SBOXOK        0x80000   /* allowed in the sandbox */
-#define CMDWIN       0x100000   /* allowed in cmdline window */
+#define CMDWIN       0x100000   /* allowed in cmdline window; when missing
+                                 * disallows editing another buffer when
+                                 * curbuf_lock is set */
 #define MODIFY       0x200000   /* forbidden in non-'modifiable' buffer */
 #define EXFLAGS      0x400000   /* allow flags after count in argument */
 #define FILES   (XFILE | EXTRA) /* multiple extra files allowed */
@@ -141,9 +143,9 @@ struct exarg {
 struct expand {
   int xp_context;               // type of expansion
   char_u *xp_pattern;           // start of item to expand
-  int xp_pattern_len;           // bytes in xp_pattern before cursor
+  size_t xp_pattern_len;        // bytes in xp_pattern before cursor
   char_u *xp_arg;               // completion function
-  int xp_scriptID;              // SID for completion function
+  sctx_T xp_script_ctx;         // SCTX for completion function
   int xp_backslash;             // one of the XP_BS_ values
 #ifndef BACKSLASH_IN_FILENAME
   int xp_shell;                 // TRUE for a shell command, more
