@@ -7,7 +7,7 @@
 
 // option_defs.h: definition of global variables for settable options
 
-// Return value from get_option_value_strict */
+// Return value from get_option_value_strict
 #define SOPT_BOOL 0x01     // Boolean option
 #define SOPT_NUM 0x02      // Number option
 #define SOPT_STRING 0x04   // String option
@@ -21,17 +21,18 @@
 #define SREQ_WIN 1     // Request window-local option value
 #define SREQ_BUF 2     // Request buffer-local option value
 
-/*
- * Default values for 'errorformat'.
- * The "%f|%l| %m" one is used for when the contents of the quickfix window is
- * written to a file.
- */
-#define DFLT_EFM \
-  "%*[^\"]\"%f\"%*\\D%l: %m,\"%f\"%*\\D%l: %m,%-G%f:%l: (Each undeclared identifier is reported only once,%-G%f:%l: for each function it appears in.),%-GIn file included from %f:%l:%c:,%-GIn file included from %f:%l:%c\\,,%-GIn file included from %f:%l:%c,%-GIn file included from %f:%l,%-G%*[ ]from %f:%l:%c,%-G%*[ ]from %f:%l:,%-G%*[ ]from %f:%l\\,,%-G%*[ ]from %f:%l,%f:%l:%c:%m,%f(%l):%m,%f:%l:%m,\"%f\"\\, line %l%*\\D%c%*[^ ] %m,%D%*\\a[%*\\d]: Entering directory %*[`']%f',%X%*\\a[%*\\d]: Leaving directory %*[`']%f',%D%*\\a: Entering directory %*[`']%f',%X%*\\a: Leaving directory %*[`']%f',%DMaking %*\\a in %f,%f|%l| %m"
+// Default values for 'errorformat'.
+// The "%f|%l| %m" one is used for when the contents of the quickfix window is
+// written to a file.
+#ifdef WIN32
+# define DFLT_EFM "%f(%l) \\=: %t%*\\D%n: %m,%*[^\"]\"%f\"%*\\D%l: %m,%f(%l) \\=: %m,%*[^ ] %f %l: %m,%f:%l:%c:%m,%f(%l):%m,%f:%l:%m,%f|%l| %m"
+#else
+# define DFLT_EFM "%*[^\"]\"%f\"%*\\D%l: %m,\"%f\"%*\\D%l: %m,%-G%f:%l: (Each undeclared identifier is reported only once,%-G%f:%l: for each function it appears in.),%-GIn file included from %f:%l:%c:,%-GIn file included from %f:%l:%c\\,,%-GIn file included from %f:%l:%c,%-GIn file included from %f:%l,%-G%*[ ]from %f:%l:%c,%-G%*[ ]from %f:%l:,%-G%*[ ]from %f:%l\\,,%-G%*[ ]from %f:%l,%f:%l:%c:%m,%f(%l):%m,%f:%l:%m,\"%f\"\\, line %l%*\\D%c%*[^ ] %m,%D%*\\a[%*\\d]: Entering directory %*[`']%f',%X%*\\a[%*\\d]: Leaving directory %*[`']%f',%D%*\\a: Entering directory %*[`']%f',%X%*\\a: Leaving directory %*[`']%f',%DMaking %*\\a in %f,%f|%l| %m"
+#endif
 
 #define DFLT_GREPFORMAT "%f:%l:%m,%f:%l%m,%f  %l%m"
 
-/* default values for b_p_ff 'fileformat' and p_ffs 'fileformats' */
+// default values for b_p_ff 'fileformat' and p_ffs 'fileformats'
 #define FF_DOS          "dos"
 #define FF_MAC          "mac"
 #define FF_UNIX         "unix"
@@ -39,7 +40,7 @@
 #ifdef USE_CRNL
 # define DFLT_FF        "dos"
 # define DFLT_FFS_VIM   "dos,unix"
-# define DFLT_FFS_VI    "dos,unix"      /* also autodetect in compatible mode */
+# define DFLT_FFS_VI    "dos,unix"      // also autodetect in compatible mode
 #else
 #  define DFLT_FF       "unix"
 #  define DFLT_FFS_VIM  "unix,dos"
@@ -47,19 +48,19 @@
 #endif
 
 
-/* Possible values for 'encoding' */
-# define ENC_UCSBOM     "ucs-bom"       /* check for BOM at start of file */
+// Possible values for 'encoding'
+# define ENC_UCSBOM     "ucs-bom"       // check for BOM at start of file
 
-/* default value for 'encoding' */
+// default value for 'encoding'
 # define ENC_DFLT       "utf-8"
 
-/* end-of-line style */
-#define EOL_UNKNOWN     -1      /* not defined yet */
-#define EOL_UNIX        0       /* NL */
-#define EOL_DOS         1       /* CR NL */
-#define EOL_MAC         2       /* CR */
+// end-of-line style
+#define EOL_UNKNOWN     -1      // not defined yet
+#define EOL_UNIX        0       // NL
+#define EOL_DOS         1       // CR NL
+#define EOL_MAC         2       // CR
 
-/* Formatting options for p_fo 'formatoptions' */
+// Formatting options for p_fo 'formatoptions'
 #define FO_WRAP         't'
 #define FO_WRAP_COMS    'c'
 #define FO_RET_COMS     'r'
@@ -70,17 +71,18 @@
 #define FO_INS_VI       'v'
 #define FO_INS_LONG     'l'
 #define FO_INS_BLANK    'b'
-#define FO_MBYTE_BREAK  'm'     /* break before/after multi-byte char */
-#define FO_MBYTE_JOIN   'M'     /* no space before/after multi-byte char */
-#define FO_MBYTE_JOIN2  'B'     /* no space between multi-byte chars */
+#define FO_MBYTE_BREAK  'm'     // break before/after multi-byte char
+#define FO_MBYTE_JOIN   'M'     // no space before/after multi-byte char
+#define FO_MBYTE_JOIN2  'B'     // no space between multi-byte chars
 #define FO_ONE_LETTER   '1'
-#define FO_WHITE_PAR    'w'     /* trailing white space continues paragr. */
-#define FO_AUTO         'a'     /* automatic formatting */
-#define FO_REMOVE_COMS  'j'     /* remove comment leaders when joining lines */
+#define FO_WHITE_PAR    'w'     // trailing white space continues paragr.
+#define FO_AUTO         'a'     // automatic formatting
+#define FO_REMOVE_COMS  'j'     // remove comment leaders when joining lines
+#define FO_PERIOD_ABBR  'p'     // don't break a single space after a period
 
 #define DFLT_FO_VI      "vt"
 #define DFLT_FO_VIM     "tcqj"
-#define FO_ALL          "tcroq2vlb1mMBn,awj"    /* for do_set() */
+#define FO_ALL          "tcroq2vlb1mMBn,awjp"   // for do_set()
 
 // characters for the p_cpo option:
 #define CPO_ALTREAD     'a'     // ":read" sets alternate file name
@@ -133,22 +135,22 @@
 #define CPO_VIM         "aABceFs_"
 #define CPO_VI          "aAbBcCdDeEfFiIJKlLmMnoOpPqrRsStuvWxXyZ$!%+>;_"
 
-/* characters for p_ww option: */
+// characters for p_ww option:
 #define WW_ALL          "bshl<>[],~"
 
-/* characters for p_mouse option: */
-#define MOUSE_NORMAL    'n'             /* use mouse in Normal mode */
-#define MOUSE_VISUAL    'v'             /* use mouse in Visual/Select mode */
-#define MOUSE_INSERT    'i'             /* use mouse in Insert mode */
-#define MOUSE_COMMAND   'c'             /* use mouse in Command-line mode */
-#define MOUSE_HELP      'h'             /* use mouse in help buffers */
-#define MOUSE_RETURN    'r'             /* use mouse for hit-return message */
-#define MOUSE_A         "nvich"         /* used for 'a' flag */
-#define MOUSE_ALL       "anvichr"       /* all possible characters */
-#define MOUSE_NONE      ' '             /* don't use Visual selection */
-#define MOUSE_NONEF     'x'             /* forced modeless selection */
+// characters for p_mouse option:
+#define MOUSE_NORMAL    'n'             // use mouse in Normal mode
+#define MOUSE_VISUAL    'v'             // use mouse in Visual/Select mode
+#define MOUSE_INSERT    'i'             // use mouse in Insert mode
+#define MOUSE_COMMAND   'c'             // use mouse in Command-line mode
+#define MOUSE_HELP      'h'             // use mouse in help buffers
+#define MOUSE_RETURN    'r'             // use mouse for hit-return message
+#define MOUSE_A         "nvich"         // used for 'a' flag
+#define MOUSE_ALL       "anvichr"       // all possible characters
+#define MOUSE_NONE      ' '             // don't use Visual selection
+#define MOUSE_NONEF     'x'             // forced modeless selection
 
-#define COCU_ALL        "nvic"          /* flags for 'concealcursor' */
+#define COCU_ALL        "nvic"          // flags for 'concealcursor'
 
 /// characters for p_shm option:
 enum {
@@ -172,57 +174,51 @@ enum {
   SHM_COMPLETIONMENU = 'c',  ///< Completion menu messages.
   SHM_RECORDING      = 'q',  ///< Short recording message.
   SHM_FILEINFO       = 'F',  ///< No file info messages.
+  SHM_SEARCHCOUNT    = 'S',  ///< Search sats: '[1/10]'
 };
 /// Represented by 'a' flag.
 #define SHM_ALL_ABBREVIATIONS ((char_u[]) { \
   SHM_RO, SHM_MOD, SHM_FILE, SHM_LAST, SHM_TEXT, SHM_LINES, SHM_NEW, SHM_WRI, \
   0, \
 })
-/// All possible flags for 'shm'.
-#define SHM_ALL ((char_u[]) { \
-  SHM_RO, SHM_MOD, SHM_FILE, SHM_LAST, SHM_TEXT, SHM_LINES, SHM_NEW, SHM_WRI, \
-  SHM_ABBREVIATIONS, SHM_WRITE, SHM_TRUNC, SHM_TRUNCALL, SHM_OVER, \
-  SHM_OVERALL, SHM_SEARCH, SHM_ATTENTION, SHM_INTRO, SHM_COMPLETIONMENU, \
-  SHM_RECORDING, SHM_FILEINFO, \
-  0, \
-})
 
-/* characters for p_go: */
-#define GO_ASEL         'a'             /* autoselect */
-#define GO_ASELML       'A'             /* autoselect modeless selection */
-#define GO_BOT          'b'             /* use bottom scrollbar */
-#define GO_CONDIALOG    'c'             /* use console dialog */
-#define GO_TABLINE      'e'             /* may show tabline */
-#define GO_FORG         'f'             /* start GUI in foreground */
-#define GO_GREY         'g'             /* use grey menu items */
-#define GO_HORSCROLL    'h'             /* flexible horizontal scrolling */
-#define GO_ICON         'i'             /* use Vim icon */
-#define GO_LEFT         'l'             /* use left scrollbar */
-#define GO_VLEFT        'L'             /* left scrollbar with vert split */
-#define GO_MENUS        'm'             /* use menu bar */
-#define GO_NOSYSMENU    'M'             /* don't source system menu */
-#define GO_POINTER      'p'             /* pointer enter/leave callbacks */
-#define GO_ASELPLUS     'P'             /* autoselectPlus */
-#define GO_RIGHT        'r'             /* use right scrollbar */
-#define GO_VRIGHT       'R'             /* right scrollbar with vert split */
-#define GO_TOOLBAR      'T'             /* add toolbar */
-#define GO_FOOTER       'F'             /* add footer */
-#define GO_VERTICAL     'v'             /* arrange dialog buttons vertically */
-#define GO_ALL          "aAbcefFghilmMprTv" /* all possible flags for 'go' */
+// characters for p_go:
+#define GO_ASEL         'a'             // autoselect
+#define GO_ASELML       'A'             // autoselect modeless selection
+#define GO_BOT          'b'             // use bottom scrollbar
+#define GO_CONDIALOG    'c'             // use console dialog
+#define GO_TABLINE      'e'             // may show tabline
+#define GO_FORG         'f'             // start GUI in foreground
+#define GO_GREY         'g'             // use grey menu items
+#define GO_HORSCROLL    'h'             // flexible horizontal scrolling
+#define GO_ICON         'i'             // use Vim icon
+#define GO_LEFT         'l'             // use left scrollbar
+#define GO_VLEFT        'L'             // left scrollbar with vert split
+#define GO_MENUS        'm'             // use menu bar
+#define GO_NOSYSMENU    'M'             // don't source system menu
+#define GO_POINTER      'p'             // pointer enter/leave callbacks
+#define GO_ASELPLUS     'P'             // autoselectPlus
+#define GO_RIGHT        'r'             // use right scrollbar
+#define GO_VRIGHT       'R'             // right scrollbar with vert split
+#define GO_TOOLBAR      'T'             // add toolbar
+#define GO_FOOTER       'F'             // add footer
+#define GO_VERTICAL     'v'             // arrange dialog buttons vertically
+#define GO_KEEPWINSIZE  'k'             // keep GUI window size
+#define GO_ALL          "aAbcefFghilmMprTvk"  // all possible flags for 'go'
 
-/* flags for 'comments' option */
-#define COM_NEST        'n'             /* comments strings nest */
-#define COM_BLANK       'b'             /* needs blank after string */
-#define COM_START       's'             /* start of comment */
-#define COM_MIDDLE      'm'             /* middle of comment */
-#define COM_END         'e'             /* end of comment */
-#define COM_AUTO_END    'x'             /* last char of end closes comment */
-#define COM_FIRST       'f'             /* first line comment only */
-#define COM_LEFT        'l'             /* left adjusted */
-#define COM_RIGHT       'r'             /* right adjusted */
-#define COM_NOBACK      'O'             /* don't use for "O" command */
-#define COM_ALL         "nbsmexflrO"    /* all flags for 'comments' option */
-#define COM_MAX_LEN     50              /* maximum length of a part */
+// flags for 'comments' option
+#define COM_NEST        'n'             // comments strings nest
+#define COM_BLANK       'b'             // needs blank after string
+#define COM_START       's'             // start of comment
+#define COM_MIDDLE      'm'             // middle of comment
+#define COM_END         'e'             // end of comment
+#define COM_AUTO_END    'x'             // last char of end closes comment
+#define COM_FIRST       'f'             // first line comment only
+#define COM_LEFT        'l'             // left adjusted
+#define COM_RIGHT       'r'             // right adjusted
+#define COM_NOBACK      'O'             // don't use for "O" command
+#define COM_ALL         "nbsmexflrO"    // all flags for 'comments' option
+#define COM_MAX_LEN     50              // maximum length of a part
 
 /// 'statusline' option flags
 enum {
@@ -277,15 +273,15 @@ enum {
   0, \
 })
 
-/* flags used for parsed 'wildmode' */
+// flags used for parsed 'wildmode'
 #define WIM_FULL        1
 #define WIM_LONGEST     2
 #define WIM_LIST        4
 
-/* arguments for can_bs() */
-#define BS_INDENT       'i'     /* "Indent" */
-#define BS_EOL          'o'     /* "eOl" */
-#define BS_START        's'     /* "Start" */
+// arguments for can_bs()
+#define BS_INDENT       'i'     // "Indent"
+#define BS_EOL          'o'     // "eOl"
+#define BS_START        's'     // "Start"
 
 #define LISPWORD_VALUE \
   "defun,define,defmacro,set!,lambda,if,case,let,flet,let*,letrec,do,do*,define-syntax,let-syntax,letrec-syntax,destructuring-bind,defpackage,defparameter,defstruct,deftype,defvar,do-all-symbols,do-external-symbols,do-symbols,dolist,dotimes,ecase,etypecase,eval-when,labels,macrolet,multiple-value-bind,multiple-value-call,multiple-value-prog1,multiple-value-setq,prog1,progv,typecase,unless,unwind-protect,when,with-input-from-string,with-open-file,with-open-stream,with-output-to-string,with-package-iterator,define-condition,handler-bind,handler-case,restart-bind,restart-case,with-simple-restart,store-value,use-value,muffle-warning,abort,continue,with-slots,with-slots*,with-accessors,with-accessors*,defclass,defmethod,print-unreadable-object"
@@ -348,20 +344,20 @@ static char *(p_bo_values[]) = {"all", "backspace", "cursor", "complete",
 #define BO_SPELL  0x20000
 #define BO_WILD   0x40000
 
-EXTERN char_u   *p_bsk;         /* 'backupskip' */
-EXTERN char_u   *p_breakat;     /* 'breakat' */
-EXTERN char_u   *p_cmp;         /* 'casemap' */
+EXTERN char_u   *p_bsk;         // 'backupskip'
+EXTERN char_u   *p_breakat;     // 'breakat'
+EXTERN char_u   *p_cmp;         // 'casemap'
 EXTERN unsigned cmp_flags;
 # ifdef IN_OPTION_C
 static char *(p_cmp_values[]) = {"internal", "keepascii", NULL};
 # endif
 # define CMP_INTERNAL           0x001
 # define CMP_KEEPASCII          0x002
-EXTERN char_u   *p_enc;         /* 'encoding' */
-EXTERN int p_deco;              /* 'delcombine' */
-EXTERN char_u   *p_ccv;         /* 'charconvert' */
-EXTERN char_u   *p_cedit;       /* 'cedit' */
-EXTERN char_u   *p_cb;          /* 'clipboard' */
+EXTERN char_u   *p_enc;         // 'encoding'
+EXTERN int p_deco;              // 'delcombine'
+EXTERN char_u   *p_ccv;         // 'charconvert'
+EXTERN char_u   *p_cedit;       // 'cedit'
+EXTERN char_u   *p_cb;          // 'clipboard'
 EXTERN unsigned cb_flags;
 #ifdef IN_OPTION_C
 static char *(p_cb_values[]) = {"unnamed", "unnamedplus", NULL};
@@ -371,29 +367,31 @@ static char *(p_cb_values[]) = {"unnamed", "unnamedplus", NULL};
 # define CB_UNNAMEDMASK         (CB_UNNAMED | CB_UNNAMEDPLUS)
 EXTERN long p_cwh;              // 'cmdwinheight'
 EXTERN long p_ch;               // 'cmdheight'
+EXTERN long p_columns;          // 'columns'
 EXTERN int p_confirm;           // 'confirm'
 EXTERN int p_cp;                // 'compatible'
 EXTERN char_u   *p_cot;         // 'completeopt'
 EXTERN long p_ph;               // 'pumheight'
+EXTERN long p_pb;               // 'pumblend'
 EXTERN char_u   *p_cpo;         // 'cpoptions'
 EXTERN char_u   *p_csprg;       // 'cscopeprg'
 EXTERN int p_csre;              // 'cscoperelative'
 EXTERN char_u   *p_csqf;        // 'cscopequickfix'
 #  define       CSQF_CMDS   "sgdctefia"
 #  define       CSQF_FLAGS  "+-0"
-EXTERN int p_cst;               /* 'cscopetag' */
-EXTERN long p_csto;             /* 'cscopetagorder' */
-EXTERN long p_cspc;             /* 'cscopepathcomp' */
-EXTERN int p_csverbose;         /* 'cscopeverbose' */
-EXTERN char_u   *p_debug;       /* 'debug' */
-EXTERN char_u   *p_def;         /* 'define' */
+EXTERN int p_cst;               // 'cscopetag'
+EXTERN long p_csto;             // 'cscopetagorder'
+EXTERN long p_cspc;             // 'cscopepathcomp'
+EXTERN int p_csverbose;         // 'cscopeverbose'
+EXTERN char_u   *p_debug;       // 'debug'
+EXTERN char_u   *p_def;         // 'define'
 EXTERN char_u   *p_inc;
-EXTERN char_u   *p_dip;         /* 'diffopt' */
-EXTERN char_u   *p_dex;         /* 'diffexpr' */
-EXTERN char_u   *p_dict;        /* 'dictionary' */
-EXTERN int p_dg;                /* 'digraph' */
-EXTERN char_u   *p_dir;         /* 'directory' */
-EXTERN char_u   *p_dy;          /* 'display' */
+EXTERN char_u   *p_dip;         // 'diffopt'
+EXTERN char_u   *p_dex;         // 'diffexpr'
+EXTERN char_u   *p_dict;        // 'dictionary'
+EXTERN int p_dg;                // 'digraph'
+EXTERN char_u   *p_dir;         // 'directory'
+EXTERN char_u   *p_dy;          // 'display'
 EXTERN unsigned dy_flags;
 #ifdef IN_OPTION_C
 static char *(p_dy_values[]) = { "lastline", "truncate", "uhex", "msgsep",
@@ -402,6 +400,7 @@ static char *(p_dy_values[]) = { "lastline", "truncate", "uhex", "msgsep",
 #define DY_LASTLINE             0x001
 #define DY_TRUNCATE             0x002
 #define DY_UHEX                 0x004
+// code should use msg_use_msgsep() to check if msgsep is active
 #define DY_MSGSEP               0x008
 EXTERN int p_ed;                // 'edcompatible'
 EXTERN int p_emoji;             // 'emoji'
@@ -463,8 +462,6 @@ EXTERN int p_hls;               // 'hlsearch'
 EXTERN long p_hi;               // 'history'
 EXTERN int p_hkmap;             // 'hkmap'
 EXTERN int p_hkmapp;            // 'hkmapp'
-EXTERN int p_fkmap;             // 'fkmap'
-EXTERN int p_altkeymap;         // 'altkeymap'
 EXTERN int p_arshape;           // 'arabicshape'
 EXTERN int p_icon;              // 'icon'
 EXTERN char_u   *p_iconstring;  // 'iconstring'
@@ -482,11 +479,11 @@ EXTERN char_u   *p_langmap;     // 'langmap'
 EXTERN int p_lnr;               // 'langnoremap'
 EXTERN int p_lrm;               // 'langremap'
 EXTERN char_u   *p_lm;          // 'langmenu'
-EXTERN long     *p_linespace;   // 'linespace'
+EXTERN long     p_lines;        // 'lines'
+EXTERN long     p_linespace;    // 'linespace'
 EXTERN char_u   *p_lispwords;   // 'lispwords'
 EXTERN long p_ls;               // 'laststatus'
 EXTERN long p_stal;             // 'showtabline'
-EXTERN char_u   *p_lcs;         // 'listchars'
 
 EXTERN int p_lz;                // 'lazyredraw'
 EXTERN int p_lpl;               // 'loadplugins'
@@ -503,6 +500,7 @@ EXTERN long p_mmd;              // 'maxmapdepth'
 EXTERN long p_mmp;              // 'maxmempattern'
 EXTERN long p_mis;              // 'menuitems'
 EXTERN char_u   *p_msm;         // 'mkspellmem'
+EXTERN long p_mle;              // 'modelineexpr'
 EXTERN long p_mls;              // 'modelines'
 EXTERN char_u   *p_mouse;       // 'mouse'
 EXTERN char_u   *p_mousem;      // 'mousemodel'
@@ -516,6 +514,23 @@ EXTERN char_u   *p_pex;         // 'patchexpr'
 EXTERN char_u   *p_pm;          // 'patchmode'
 EXTERN char_u   *p_path;        // 'path'
 EXTERN char_u   *p_cdpath;      // 'cdpath'
+EXTERN long p_pyx;              // 'pyxversion'
+EXTERN char_u *p_rdb;           // 'redrawdebug'
+EXTERN unsigned rdb_flags;
+# ifdef IN_OPTION_C
+static char *(p_rdb_values[]) = {
+  "compositor",
+  "nothrottle",
+  "invalid",
+  "nodelta",
+  NULL
+};
+# endif
+# define RDB_COMPOSITOR         0x001
+# define RDB_NOTHROTTLE         0x002
+# define RDB_INVALID            0x004
+# define RDB_NODELTA            0x008
+
 EXTERN long p_rdt;              // 'redrawtime'
 EXTERN int p_remap;             // 'remap'
 EXTERN long p_re;               // 'regexpengine'
@@ -538,12 +553,12 @@ EXTERN char_u   *p_slm;         // 'selectmode'
 EXTERN char_u   *p_ssop;        // 'sessionoptions'
 EXTERN unsigned ssop_flags;
 # ifdef IN_OPTION_C
-/* Also used for 'viewoptions'! */
-static char *(p_ssop_values[]) = {"buffers", "winpos", "resize", "winsize",
-                                  "localoptions", "options", "help", "blank",
-                                  "globals", "slash", "unix",
-                                  "sesdir", "curdir", "folds", "cursor",
-                                  "tabpages", NULL };
+// Also used for 'viewoptions'!
+static char *(p_ssop_values[]) = {
+  "buffers", "winpos", "resize", "winsize",
+  "localoptions", "options", "help", "blank", "globals", "slash", "unix",
+  "sesdir", "curdir", "folds", "cursor", "tabpages", NULL
+};
 # endif
 # define SSOP_BUFFERS           0x001
 # define SSOP_WINPOS            0x002
@@ -637,8 +652,8 @@ EXTERN long p_ul;               ///< 'undolevels'
 EXTERN long p_ur;               ///< 'undoreload'
 EXTERN long p_uc;               ///< 'updatecount'
 EXTERN long p_ut;               ///< 'updatetime'
-EXTERN char_u *p_fcs;           ///< 'fillchar'
 EXTERN char_u *p_shada;         ///< 'shada'
+EXTERN char *p_shadafile;       ///< 'shadafile'
 EXTERN char_u *p_vdir;          ///< 'viewdir'
 EXTERN char_u *p_vop;           ///< 'viewoptions'
 EXTERN unsigned vop_flags;      ///< uses SSOP_ flags
@@ -648,18 +663,24 @@ EXTERN unsigned ve_flags;
 # ifdef IN_OPTION_C
 static char *(p_ve_values[]) = {"block", "insert", "all", "onemore", NULL};
 # endif
-# define VE_BLOCK       5       /* includes "all" */
-# define VE_INSERT      6       /* includes "all" */
+# define VE_BLOCK       5       // includes "all"
+# define VE_INSERT      6       // includes "all"
 # define VE_ALL         4
 # define VE_ONEMORE     8
-EXTERN long p_verbose;          /* 'verbose' */
+EXTERN long p_verbose;          // 'verbose'
 #ifdef IN_OPTION_C
-char_u  *p_vfile = (char_u *)""; /* used before options are initialized */
+char_u *p_vfile = (char_u *)"";   // used before options are initialized
 #else
-extern char_u   *p_vfile;       /* 'verbosefile' */
+extern char_u   *p_vfile;       // 'verbosefile'
 #endif
 EXTERN int p_warn;              // 'warn'
 EXTERN char_u   *p_wop;         // 'wildoptions'
+EXTERN unsigned wop_flags;
+# ifdef IN_OPTION_C
+static char *(p_wop_values[]) =  { "tagfile", "pum", NULL };
+#endif
+#define WOP_TAGFILE             0x01
+#define WOP_PUM                 0x02
 EXTERN long p_window;           // 'window'
 EXTERN char_u   *p_wak;         // 'winaltkeys'
 EXTERN char_u   *p_wig;         // 'wildignore'
@@ -767,7 +788,7 @@ enum {
   , BV_UDF
   , BV_UL
   , BV_WM
-  , BV_COUNT        /* must be the last one */
+  , BV_COUNT        // must be the last one
 };
 
 /*
@@ -813,17 +834,20 @@ enum {
   , WV_WRAP
   , WV_SCL
   , WV_WINHL
+  , WV_FCS
+  , WV_LCS
+  , WV_WINBL
   , WV_COUNT        // must be the last one
 };
 
-/* Value for b_p_ul indicating the global value must be used. */
+// Value for b_p_ul indicating the global value must be used.
 #define NO_LOCAL_UNDOLEVEL -123456
 
 #define SB_MAX 100000  // Maximum 'scrollback' value.
 
 /// Stores an identifier of a script or channel that last set an option.
 typedef struct {
-  scid_T script_id;        /// Script ID or one of SID_* special values.
+  sctx_T script_ctx;       /// script context where the option was last set
   uint64_t channel_id;     /// Only used when script_id is SID_API_CLIENT.
 } LastSet;
 
