@@ -8,6 +8,7 @@
 --    defaults={condition=nil, if_true={vi=224, vim=0}, if_false=nil},
 --    secure=nil, gettext=nil, noglob=nil, normal_fname_chars=nil,
 --    pri_mkrc=nil, deny_in_modelines=nil, normal_dname_chars=nil,
+--    modelineexpr=nil,
 --    expand=nil, nodefault=nil, no_mkrc=nil, vi_def=true, vim=true,
 --    alloced=nil,
 --    save_pv_indir=nil,
@@ -39,11 +40,13 @@ local imacros=function(s)
     return '(intptr_t)' .. s
   end
 end
-local N_=function(s)
+local N_=function(s) -- luacheck: ignore 211 (currently unused)
   return function()
     return 'N_(' .. cstr(s) .. ')'
   end
 end
+-- used for 'cinkeys' and 'indentkeys'
+local indentkeys_default = '0{,0},0),0],:,0#,!^F,o,O,e';
 return {
   cstr=cstr,
   options={
@@ -81,13 +84,6 @@ return {
       varname='p_ari',
       defaults={if_true={vi=false}}
     },
-    {
-      full_name='altkeymap', abbreviation='akm',
-      type='bool', scope={'global'},
-      vi_def=true,
-      varname='p_altkeymap',
-      defaults={if_true={vi=false}}
-    },
     {
       full_name='ambiwidth', abbreviation='ambw',
       type='string', scope={'global'},
@@ -186,14 +182,15 @@ return {
     {
       full_name='backupskip', abbreviation='bsk',
       type='string', list='onecomma', scope={'global'},
+      deny_duplicates=true,
       vi_def=true,
       varname='p_bsk',
       defaults={if_true={vi=""}}
     },
     {
       full_name='belloff', abbreviation='bo',
-      deny_duplicates=true,
       type='string', list='comma', scope={'global'},
+      deny_duplicates=true,
       vi_def=true,
       varname='p_bo',
       defaults={if_true={vi="all"}}
@@ -286,6 +283,7 @@ return {
       deny_duplicates=true,
       vi_def=true,
       expand=true,
+      secure=true,
       varname='p_cdpath',
       defaults={if_true={vi=",,"}}
     },
@@ -326,7 +324,7 @@ return {
       vi_def=true,
       alloced=true,
       varname='p_cink',
-      defaults={if_true={vi="0{,0},0),:,0#,!^F,o,O,e"}}
+      defaults={if_true={vi=indentkeys_default}}
     },
     {
       full_name='cinoptions', abbreviation='cino',
@@ -381,10 +379,9 @@ return {
       full_name='columns', abbreviation='co',
       type='number', scope={'global'},
       no_mkrc=true,
-      nodefault=true,
       vi_def=true,
       redraw={'everything'},
-      varname='Columns',
+      varname='p_columns',
       defaults={if_true={vi=macros('DFLT_COLS')}}
     },
     {
@@ -546,7 +543,7 @@ return {
       full_name='cursorcolumn', abbreviation='cuc',
       type='bool', scope={'window'},
       vi_def=true,
-      redraw={'current_window'},
+      redraw={'current_window_only'},
       defaults={if_true={vi=false}}
     },
     {
@@ -761,6 +758,7 @@ return {
     {
       full_name='fileencodings', abbreviation='fencs',
       type='string', list='onecomma', scope={'global'},
+      deny_duplicates=true,
       vi_def=true,
       varname='p_fencs',
       defaults={if_true={vi="ucs-bom,utf-8,default,latin1"}}
@@ -806,11 +804,11 @@ return {
     },
     {
       full_name='fillchars', abbreviation='fcs',
-      type='string', list='onecomma', scope={'global'},
+      type='string', list='onecomma', scope={'window'},
       deny_duplicates=true,
       vi_def=true,
-      redraw={'all_windows'},
-      varname='p_fcs',
+      alloced=true,
+      redraw={'current_window'},
       defaults={if_true={vi=''}}
     },
     {
@@ -821,13 +819,6 @@ return {
       varname='p_fixeol',
       defaults={if_true={vi=true}}
     },
-    {
-      full_name='fkmap', abbreviation='fk',
-      type='bool', scope={'global'},
-      vi_def=true,
-      varname='p_fkmap',
-      defaults={if_true={vi=false}}
-    },
     {
       full_name='foldclose', abbreviation='fcl',
       type='string', list='onecomma', scope={'global'},
@@ -856,6 +847,7 @@ return {
       type='string', scope={'window'},
       vi_def=true,
       vim=true,
+      modelineexpr=true,
       alloced=true,
       redraw={'current_window'},
       defaults={if_true={vi="0"}}
@@ -931,6 +923,7 @@ return {
       type='string', scope={'window'},
       vi_def=true,
       vim=true,
+      modelineexpr=true,
       alloced=true,
       redraw={'current_window'},
       defaults={if_true={vi="foldtext()"}}
@@ -940,6 +933,7 @@ return {
       type='string', scope={'buffer'},
       vi_def=true,
       vim=true,
+      modelineexpr=true,
       alloced=true,
       varname='p_fex',
       defaults={if_true={vi=""}}
@@ -1028,6 +1022,7 @@ return {
     {
       full_name='guifontset', abbreviation='gfs',
       type='string', list='onecomma', scope={'global'},
+      deny_duplicates=true,
       vi_def=true,
       varname='p_guifontset',
       redraw={'ui_option'},
@@ -1053,6 +1048,7 @@ return {
       full_name='guitablabel', abbreviation='gtl',
       type='string', scope={'global'},
       vi_def=true,
+      modelineexpr=true,
       redraw={'current_window'},
       enable_if=false,
     },
@@ -1082,6 +1078,7 @@ return {
     {
       full_name='helplang', abbreviation='hlg',
       type='string', list='onecomma', scope={'global'},
+      deny_duplicates=true,
       vi_def=true,
       varname='p_hlg',
       defaults={if_true={vi=""}}
@@ -1143,6 +1140,7 @@ return {
       full_name='iconstring',
       type='string', scope={'global'},
       vi_def=true,
+      modelineexpr=true,
       varname='p_iconstring',
       defaults={if_true={vi=""}}
     },
@@ -1173,9 +1171,7 @@ return {
       vi_def=true,
       varname='p_iminsert', pv_name='p_imi',
       defaults={
-        condition='B_IMODE_IM',
-        if_true={vi=macros('B_IMODE_IM')},
-        if_false={vi=macros('B_IMODE_NONE')},
+        if_true={vi=macros('B_IMODE_NONE')},
       }
     },
     {
@@ -1184,9 +1180,7 @@ return {
       vi_def=true,
       varname='p_imsearch', pv_name='p_ims',
       defaults={
-        condition='B_IMODE_IM',
-        if_true={vi=macros('B_IMODE_IM')},
-        if_false={vi=macros('B_IMODE_NONE')},
+        if_true={vi=macros('B_IMODE_USE_INSERT')},
       }
     },
     {
@@ -1209,6 +1203,7 @@ return {
       full_name='includeexpr', abbreviation='inex',
       type='string', scope={'buffer'},
       vi_def=true,
+      modelineexpr=true,
       alloced=true,
       varname='p_inex',
       defaults={if_true={vi=""}}
@@ -1225,6 +1220,7 @@ return {
       type='string', scope={'buffer'},
       vi_def=true,
       vim=true,
+      modelineexpr=true,
       alloced=true,
       varname='p_inde',
       defaults={if_true={vi=""}}
@@ -1236,7 +1232,7 @@ return {
       vi_def=true,
       alloced=true,
       varname='p_indk',
-      defaults={if_true={vi="0{,0},:,0#,!^F,o,O,e"}}
+      defaults={if_true={vi=indentkeys_default}}
     },
     {
       full_name='infercase', abbreviation='inf',
@@ -1387,10 +1383,9 @@ return {
       full_name='lines',
       type='number', scope={'global'},
       no_mkrc=true,
-      nodefault=true,
       vi_def=true,
       redraw={'everything'},
-      varname='Rows',
+      varname='p_lines',
       defaults={if_true={vi=macros('DFLT_ROWS')}}
     },
     {
@@ -1425,11 +1420,11 @@ return {
     },
     {
       full_name='listchars', abbreviation='lcs',
-      type='string', list='onecomma', scope={'global'},
+      type='string', list='onecomma', scope={'window'},
       deny_duplicates=true,
       vim=true,
-      redraw={'all_windows'},
-      varname='p_lcs',
+      alloced=true,
+      redraw={'current_window'},
       defaults={if_true={vi="eol:$", vim="tab:> ,trail:-,nbsp:+"}}
     },
     {
@@ -1538,6 +1533,14 @@ return {
       varname='p_ml',
       defaults={if_true={vi=false, vim=true}}
     },
+    {
+      full_name='modelineexpr', abbreviation='mle',
+      type='bool', scope={'global'},
+      vi_def=true,
+      secure=true,
+      varname='p_mle',
+      defaults={if_true={vi=false}}
+    },
     {
       full_name='modelines', abbreviation='mls',
       type='number', scope={'global'},
@@ -1809,6 +1812,22 @@ return {
       varname='p_ph',
       defaults={if_true={vi=0}}
     },
+    {
+      full_name='pumblend', abbreviation='pb',
+      type='number', scope={'global'},
+      vi_def=true,
+      redraw={'ui_option'},
+      varname='p_pb',
+      defaults={if_true={vi=0}}
+    },
+    {
+      full_name='pyxversion', abbreviation='pyx',
+      type='number', scope={'global'},
+      secure=true,
+      vi_def=true,
+      varname='p_pyx',
+      defaults={if_true={vi=0}}
+    },
     {
       full_name='quoteescape', abbreviation='qe',
       type='string', scope={'buffer'},
@@ -1826,6 +1845,13 @@ return {
       varname='p_ro',
       defaults={if_true={vi=false}}
     },
+    {
+      full_name='redrawdebug', abbreviation='rdb',
+      type='string', list='onecomma', scope={'global'},
+      vi_def=true,
+      varname='p_rdb',
+      defaults={if_true={vi=''}}
+    },
     {
       full_name='redrawtime', abbreviation='rdt',
       type='number', scope={'global'},
@@ -1898,6 +1924,7 @@ return {
       type='string', scope={'global'},
       vi_def=true,
       alloced=true,
+      modelineexpr=true,
       redraw={'statuslines'},
       varname='p_ruf',
       defaults={if_true={vi=""}}
@@ -1926,7 +1953,7 @@ return {
       vi_def=true,
       varname='p_scbk',
       redraw={'current_buffer'},
-      defaults={if_true={vi=10000}}
+      defaults={if_true={vi=-1}}
     },
     {
       full_name='scrollbind', abbreviation='scb',
@@ -2009,6 +2036,15 @@ return {
       varname='p_shada',
       defaults={if_true={vi="", vim="!,'100,<50,s10,h"}}
     },
+    {
+      full_name='shadafile', abbreviation='sdf',
+      type='string', list='onecomma', scope={'global'},
+      deny_duplicates=true,
+      vi_def=true,
+      secure=true,
+      varname='p_shadafile',
+      defaults={if_true={vi=""}}
+    },
     {
       full_name='shell', abbreviation='sh',
       type='string', scope={'global'},
@@ -2120,7 +2156,7 @@ return {
       type='string', list='flags', scope={'global'},
       vim=true,
       varname='p_shm',
-      defaults={if_true={vi="", vim="filnxtToOF"}}
+      defaults={if_true={vi="S", vim="filnxtToOF"}}
     },
     {
       full_name='showbreak', abbreviation='sbr',
@@ -2240,6 +2276,7 @@ return {
     {
       full_name='spellfile', abbreviation='spf',
       type='string', list='onecomma', scope={'buffer'},
+      deny_duplicates=true,
       secure=true,
       vi_def=true,
       alloced=true,
@@ -2250,6 +2287,7 @@ return {
     {
       full_name='spelllang', abbreviation='spl',
       type='string', list='onecomma', scope={'buffer'},
+      deny_duplicates=true,
       vi_def=true,
       alloced=true,
       expand=true,
@@ -2260,6 +2298,7 @@ return {
     {
       full_name='spellsuggest', abbreviation='sps',
       type='string', list='onecomma', scope={'global'},
+      deny_duplicates=true,
       secure=true,
       vi_def=true,
       expand=true,
@@ -2293,6 +2332,7 @@ return {
       type='string', scope={'global', 'window'},
       vi_def=true,
       alloced=true,
+      modelineexpr=true,
       redraw={'statuslines'},
       varname='p_stl',
       defaults={if_true={vi=""}}
@@ -2352,6 +2392,7 @@ return {
       full_name='tabline', abbreviation='tal',
       type='string', scope={'global'},
       vi_def=true,
+      modelineexpr=true,
       redraw={'all_windows'},
       varname='p_tal',
       defaults={if_true={vi=""}}
@@ -2511,6 +2552,7 @@ return {
       full_name='titlestring',
       type='string', scope={'global'},
       vi_def=true,
+      modelineexpr=true,
       varname='p_titlestring',
       defaults={if_true={vi=""}}
     },
@@ -2617,12 +2659,14 @@ return {
       defaults={if_true={vi="folds,options,cursor,curdir"}}
     },
     {
+      -- Alias for "shada".
       full_name='viminfo', abbreviation='vi',
-      type='string', list='onecomma', scope={'global'},
-      deny_duplicates=true,
-      secure=true,
-      varname='p_shada',
-      defaults={if_true={vi="", vim="!,'100,<50,s10,h"}}
+      type='string', scope={'global'}, nodefault=true,
+    },
+    {
+      -- Alias for "shadafile".
+      full_name='viminfofile', abbreviation='vif',
+      type='string', scope={'global'}, nodefault=true,
     },
     {
       full_name='virtualedit', abbreviation='ve',
@@ -2701,10 +2745,11 @@ return {
     },
     {
       full_name='wildoptions', abbreviation='wop',
-      type='string', scope={'global'},
-      vi_def=true,
+      type='string', list='onecomma', scope={'global'},
+      deny_duplicates=true,
+      vim=true,
       varname='p_wop',
-      defaults={if_true={vi=""}}
+      defaults={if_true={vi='', vim='pum,tagfile'}}
     },
     {
       full_name='winaltkeys', abbreviation='wak',
@@ -2713,6 +2758,13 @@ return {
       varname='p_wak',
       defaults={if_true={vi="menu"}}
     },
+    {
+      full_name='winblend', abbreviation='winbl',
+      type='number', scope={'window'},
+      vi_def=true,
+      redraw={'current_window'},
+      defaults={if_true={vi=0}}
+    },
     {
       full_name='winhighlight', abbreviation='winhl',
       type='string', scope={'window'},
