@@ -21,11 +21,6 @@ typedef struct {
   uv_dirent_t ent;  ///< @private The entry information.
 } Directory;
 
-/// Function to convert libuv error to char * error description
-///
-/// negative libuv error codes are returned by a number of os functions.
-#define os_strerror uv_strerror
-
 // Values returned by os_nodetype()
 #define NODE_NORMAL     0  // file or directory, check with os_isdir()
 #define NODE_WRITABLE   1  // something we can write to (character
