@@ -94,10 +94,8 @@ typedef struct {
   char_u program[1];                    /* actually longer.. */
 } bt_regprog_T;
 
-/*
- * Structure representing a NFA state.
- * A NFA state may have no outgoing edge, when it is a NFA_MATCH state.
- */
+// Structure representing a NFA state.
+// An NFA state may have no outgoing edge, when it is a NFA_MATCH state.
 typedef struct nfa_state nfa_state_T;
 struct nfa_state {
   int c;
@@ -156,11 +154,11 @@ struct reg_extmatch {
 };
 
 struct regengine {
-  regprog_T   *(*regcomp)(char_u*, int);
+  regprog_T   *(*regcomp)(char_u *, int);
   void (*regfree)(regprog_T *);
-  int (*regexec_nl)(regmatch_T*, char_u*, colnr_T, bool);
-  long (*regexec_multi)(regmmatch_T*, win_T*, buf_T*, linenr_T, colnr_T,
-      proftime_T*);
+  int (*regexec_nl)(regmatch_T *, char_u *, colnr_T, bool);
+  long (*regexec_multi)(regmmatch_T *, win_T *, buf_T *, linenr_T, colnr_T,
+                        proftime_T *, int *);
   char_u      *expr;
 };
 
