@@ -14,9 +14,13 @@ typedef enum {
   kUIPopupmenu,
   kUITabline,
   kUIWildmenu,
-#define kUIGlobalCount (kUIWildmenu+1)
+  kUIMessages,
+#define kUIGlobalCount kUILinegrid
   kUILinegrid,
+  kUIMultigrid,
   kUIHlState,
+  kUITermColors,
+  kUIFloatDebug,
   kUIExtCount,
 } UIExtension;
 
@@ -25,31 +29,37 @@ EXTERN const char *ui_ext_names[] INIT(= {
   "ext_popupmenu",
   "ext_tabline",
   "ext_wildmenu",
+  "ext_messages",
   "ext_linegrid",
+  "ext_multigrid",
   "ext_hlstate",
+  "ext_termcolors",
+  "_debug_float",
 });
 
-
 typedef struct ui_t UI;
 
+enum {
+  kLineFlagWrap = 1,
+  kLineFlagInvalid = 2,
+};
+
+typedef int LineFlags;
+
 struct ui_t {
   bool rgb;
-  bool ui_ext[kUIExtCount];  ///< Externalized widgets
-  int width, height;
+  bool override;  ///< Force highest-requested UI capabilities.
+  bool composed;
+  bool ui_ext[kUIExtCount];  ///< Externalized UI capabilities.
+  int width;
+  int height;
+  int pum_height;
   void *data;
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "ui_events.generated.h"
 #endif
 
-  // For perfomance and simplicity, we use the dense screen representation
-  // in the bridge and the TUI. The remote_ui module will translate this
-  // in to the public grid_line format.
-  void (*raw_line)(UI *ui, Integer grid, Integer row, Integer startcol,
-                   Integer endcol, Integer clearcol, Integer clearattr,
-                   Boolean wrap, const schar_T *chunk, const sattr_T *attrs);
-  void (*event)(UI *ui, char *name, Array args, bool *args_consumed);
-  void (*stop)(UI *ui);
   void (*inspect)(UI *ui, Dictionary *info);
 };
 
@@ -57,4 +67,7 @@ struct ui_t {
 # include "ui.h.generated.h"
 # include "ui_events_call.h.generated.h"
 #endif
+
+
+EXTERN MultiQueue *resize_events;
 #endif  // NVIM_UI_H
