@@ -155,6 +155,8 @@ enum {
   EXPAND_USER_ADDR_TYPE,
   EXPAND_PACKADD,
   EXPAND_MESSAGES,
+  EXPAND_MAPCLEAR,
+  EXPAND_ARGLIST,
   EXPAND_CHECKHEALTH,
 };
 
@@ -207,7 +209,7 @@ enum { FOLD_TEXT_LEN = 51 };  //!< buffer size for get_foldtext()
 
 
 // defines to avoid typecasts from (char_u *) to (char *) and back
-// (vim_strchr() and vim_strrchr() are now in alloc.c)
+// (vim_strchr() is now in strings.c)
 
 #define STRLEN(s)           strlen((char *)(s))
 #define STRCPY(d, s)        strcpy((char *)(d), (char *)(s))
@@ -238,6 +240,8 @@ enum { FOLD_TEXT_LEN = 51 };  //!< buffer size for get_foldtext()
 # endif
 #endif
 
+#define STRRCHR(s, c)       (char_u *)strrchr((const char *)(s), (c))
+
 #define STRCAT(d, s)        strcat((char *)(d), (char *)(s))
 #define STRNCAT(d, s, n)    strncat((char *)(d), (char *)(s), (size_t)(n))
 #define STRLCAT(d, s, n)    xstrlcat((char *)(d), (char *)(s), (size_t)(n))
@@ -273,7 +277,6 @@ enum { FOLD_TEXT_LEN = 51 };  //!< buffer size for get_foldtext()
 
 // Enums need a typecast to be used as array index (for Ultrix).
 #define HL_ATTR(n)      highlight_attr[(int)(n)]
-#define TERM_STR(n)     term_strings[(int)(n)]
 
 /// Maximum number of bytes in a multi-byte character.  It can be one 32-bit
 /// character of up to 6 bytes, or one 16-bit character of up to three bytes
@@ -284,17 +287,15 @@ enum { FOLD_TEXT_LEN = 51 };  //!< buffer size for get_foldtext()
 // functions of these names. The declarations would break if the defines had
 // been seen at that stage.  But it must be before globals.h, where error_ga
 // is declared.
-#define mch_errmsg(str)        fprintf(stderr, "%s", (str))
-#define display_errors()       fflush(stderr)
-#define mch_msg(str)           printf("%s", (str))
+#ifndef WIN32
+# define mch_errmsg(str)        fprintf(stderr, "%s", (str))
+# define mch_msg(str)           printf("%s", (str))
+#endif
 
 #include "nvim/globals.h"        // global variables and messages
 #include "nvim/buffer_defs.h"    // buffer and windows
 #include "nvim/ex_cmds_defs.h"   // Ex command defines
 
-# define SET_NO_HLSEARCH(flag) no_hlsearch = (flag); set_vim_var_nr( \
-    VV_HLSEARCH, !no_hlsearch && p_hls)
-
 // Used for flags in do_in_path()
 #define DIP_ALL 0x01    // all matches, not just the first one
 #define DIP_DIR 0x02    // find directories instead of files
