@@ -1,23 +1,21 @@
 local helpers = require('test.functional.helpers')(after_each)
 local lfs = require('lfs')
-local command, eq, neq, spawn, nvim_prog, set_session, write_file =
-  helpers.command, helpers.eq, helpers.neq, helpers.spawn,
-  helpers.nvim_prog, helpers.set_session, helpers.write_file
+local clear = helpers.clear
+local command, eq, neq, write_file =
+  helpers.command, helpers.eq, helpers.neq, helpers.write_file
 local iswin = helpers.iswin
 local read_file = helpers.read_file
 
 describe(':wshada', function()
   local shada_file = 'wshada_test'
-  local session
 
   before_each(function()
-    -- Override the default session because we need 'swapfile' for these tests.
-    session = spawn({nvim_prog, '-u', 'NONE', '-i', iswin() and 'nul' or '/dev/null', '--embed',
-                           '--cmd', 'set swapfile'})
-    set_session(session)
+    clear{args={'-i', iswin() and 'nul' or '/dev/null',
+                -- Need 'swapfile' for these tests.
+                '--cmd', 'set swapfile undodir=. directory=. viewdir=. backupdir=. belloff= noshowcmd noruler'},
+          args_rm={'-n', '-i', '--cmd'}}
   end)
   after_each(function ()
-    session:close()
     os.remove(shada_file)
   end)
 
