@@ -18,6 +18,15 @@ describe('assert function:', function()
     clear()
   end)
 
+  describe('assert_beeps', function()
+    it('works', function()
+      call('assert_beeps', 'normal h')
+      expected_empty()
+      call('assert_beeps', 'normal 0')
+      expected_errors({'command did not beep: normal 0'})
+    end)
+  end)
+
   -- assert_equal({expected}, {actual}, [, {msg}])
   describe('assert_equal', function()
     it('should not change v:errors when expected is equal to actual', function()
