@@ -15,6 +15,7 @@ local neq = helpers.neq
 local mkdir = helpers.mkdir
 local rmdir = helpers.rmdir
 local alter_slashes = helpers.alter_slashes
+local tbl_contains = helpers.tbl_contains
 
 describe('startup defaults', function()
   describe(':filetype', function()
@@ -160,20 +161,45 @@ describe('startup defaults', function()
     end)
   end)
 
-  describe("'packpath'", function()
-    it('defaults to &runtimepath', function()
-      eq(meths.get_option('runtimepath'), meths.get_option('packpath'))
-    end)
+  it("'shadafile' ('viminfofile')", function()
+    local env = {XDG_DATA_HOME='Xtest-userdata', XDG_CONFIG_HOME='Xtest-userconfig'}
+    clear{args={}, args_rm={'-i'}, env=env}
+    -- Default 'shadafile' is empty.
+    -- This means use the default location. :help shada-file-name
+    eq('', meths.get_option('shadafile'))
+    eq('', meths.get_option('viminfofile'))
+    -- Check that shada data (such as v:oldfiles) is saved/restored.
+    command('edit Xtest-foo')
+    command('write')
+    local f = eval('fnamemodify(@%,":p")')
+    assert(string.len(f) > 3)
+    command('qall')
+    clear{args={}, args_rm={'-i'}, env=env}
+    eq({ f }, eval('v:oldfiles'))
+    os.remove('Xtest-foo')
+    rmdir('Xtest-userdata')
 
-    it('does not follow modifications to runtimepath', function()
-      meths.command('set runtimepath+=foo')
-      neq(meths.get_option('runtimepath'), meths.get_option('packpath'))
-      meths.command('set packpath+=foo')
-      eq(meths.get_option('runtimepath'), meths.get_option('packpath'))
-    end)
+    -- Handles viminfo/viminfofile as alias for shada/shadafile.
+    eq('\n  shadafile=', eval('execute("set shadafile?")'))
+    eq('\n  shadafile=', eval('execute("set viminfofile?")'))
+    eq("\n  shada=!,'100,<50,s10,h", eval('execute("set shada?")'))
+    eq("\n  shada=!,'100,<50,s10,h", eval('execute("set viminfo?")'))
+  end)
+
+  it("'packpath'", function()
+    clear()
+    -- Defaults to &runtimepath.
+    eq(meths.get_option('runtimepath'), meths.get_option('packpath'))
+
+    -- Does not follow modifications to runtimepath.
+    meths.command('set runtimepath+=foo')
+    neq(meths.get_option('runtimepath'), meths.get_option('packpath'))
+    meths.command('set packpath+=foo')
+    eq(meths.get_option('runtimepath'), meths.get_option('packpath'))
   end)
 
   it('v:progpath is set to the absolute path', function()
+    clear()
     eq(eval("fnamemodify(v:progpath, ':p')"), eval('v:progpath'))
   end)
 
@@ -231,6 +257,23 @@ describe('XDG-based defaults', function()
   -- Need separate describe() blocks to not run clear() twice.
   -- Do not put before_each() here for the same reasons.
 
+  it("&runtimepath data-dir matches stdpath('data') #9910", function()
+    clear()
+    local rtp = eval('split(&runtimepath, ",")')
+    local rv = {}
+    local expected = (iswin()
+                      and { [[\nvim-data\site]], [[\nvim-data\site\after]], }
+                      or { '/nvim/site', '/nvim/site/after', })
+
+    for _,v in ipairs(rtp) do
+      local m = string.match(v, [=[[/\]nvim[^/\]*[/\]site.*$]=])
+      if m and not tbl_contains(rv, m) then
+        table.insert(rv, m)
+      end
+    end
+    eq(expected, rv)
+  end)
+
   describe('with empty/broken environment', function()
     it('sets correct defaults', function()
       clear({env={
