@@ -494,7 +494,7 @@ describe('Command-line coloring', function()
       {EOB:~                                       }|
       {EOB:~                                       }|
       {EOB:~                                       }|
-      Type  :qa!  and pr...nges and exit Nvim |
+      Type  :qa  and pre...nter> to exit Nvim |
     ]])
   end)
   it('works fine with NUL, NL, CR', function()
@@ -975,8 +975,6 @@ describe('Expressions coloring support', function()
     ]])
     funcs.setreg('a', {'\192'})
     feed('<C-r>="<C-r><C-r>a"<C-r><C-r>a"foo"')
-    -- TODO(ZyX-I): Parser highlighting should not override special character
-    --              highlighting.
     screen:expect([[
                                               |
       {EOB:~                                       }|
