@@ -30,6 +30,24 @@ describe('mappings', function()
     add_mapping('<c-s-a-d-up>', '<c-s-a-d-up>')
     add_mapping('<c-d-a>', '<c-d-a>')
     add_mapping('<d-1>', '<d-1>')
+    add_mapping('<khome>','<khome>')
+    add_mapping('<kup>','<kup>')
+    add_mapping('<kpageup>','<kpageup>')
+    add_mapping('<kleft>','<kleft>')
+    add_mapping('<korigin>','<korigin>')
+    add_mapping('<kright>','<kright>')
+    add_mapping('<kend>','<kend>')
+    add_mapping('<kdown>','<kdown>')
+    add_mapping('<kpagedown>','<kpagedown>')
+    add_mapping('<kinsert>','<kinsert>')
+    add_mapping('<kdel>','<kdel>')
+    add_mapping('<kdivide>','<kdivide>')
+    add_mapping('<kmultiply>','<kmultiply>')
+    add_mapping('<kminus>','<kminus>')
+    add_mapping('<kplus>','<kplus>')
+    add_mapping('<kenter>','<kenter>')
+    add_mapping('<kcomma>','<kcomma>')
+    add_mapping('<kequal>','<kequal>')
   end)
 
   it('ok', function()
@@ -48,77 +66,47 @@ describe('mappings', function()
     check_mapping('<c-d-a>', '<c-d-a>')
     check_mapping('<d-c-a>', '<c-d-a>')
     check_mapping('<d-1>', '<d-1>')
-  end)
-end)
-
-describe('feeding large chunks of input with <Paste>', function()
-  local screen
-  before_each(function()
-    clear()
-    screen = Screen.new()
-    screen:attach()
-    feed_command('set ruler')
+    check_mapping('<khome>','<khome>')
+    check_mapping('<KP7>','<khome>')
+    check_mapping('<kup>','<kup>')
+    check_mapping('<KP8>','<kup>')
+    check_mapping('<kpageup>','<kpageup>')
+    check_mapping('<KP9>','<kpageup>')
+    check_mapping('<kleft>','<kleft>')
+    check_mapping('<KP4>','<kleft>')
+    check_mapping('<korigin>','<korigin>')
+    check_mapping('<KP5>','<korigin>')
+    check_mapping('<kright>','<kright>')
+    check_mapping('<KP6>','<kright>')
+    check_mapping('<kend>','<kend>')
+    check_mapping('<KP1>','<kend>')
+    check_mapping('<kdown>','<kdown>')
+    check_mapping('<KP2>','<kdown>')
+    check_mapping('<kpagedown>','<kpagedown>')
+    check_mapping('<KP3>','<kpagedown>')
+    check_mapping('<kinsert>','<kinsert>')
+    check_mapping('<KP0>','<kinsert>')
+    check_mapping('<kdel>','<kdel>')
+    check_mapping('<KPPeriod>','<kdel>')
+    check_mapping('<kdivide>','<kdivide>')
+    check_mapping('<KPDiv>','<kdivide>')
+    check_mapping('<kmultiply>','<kmultiply>')
+    check_mapping('<KPMult>','<kmultiply>')
+    check_mapping('<kminus>','<kminus>')
+    check_mapping('<KPMinus>','<kminus>')
+    check_mapping('<kplus>','<kplus>')
+    check_mapping('<KPPlus>','<kplus>')
+    check_mapping('<kenter>','<kenter>')
+    check_mapping('<KPEnter>','<kenter>')
+    check_mapping('<kcomma>','<kcomma>')
+    check_mapping('<KPComma>','<kcomma>')
+    check_mapping('<kequal>','<kequal>')
+    check_mapping('<KPEquals>','<kequal>')
   end)
 
-  it('ok', function()
-    if helpers.skip_fragile(pending) then
-      return
-    end
-    local t = {}
-    for i = 1, 20000 do
-      t[i] = 'item ' .. tostring(i)
-    end
-    feed('i<Paste>')
-    screen:expect([[
-      ^                                                     |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      -- INSERT (paste) --                                 |
-    ]])
-    feed(table.concat(t, '<Enter>'))
-    screen:expect([[
-      item 19988                                           |
-      item 19989                                           |
-      item 19990                                           |
-      item 19991                                           |
-      item 19992                                           |
-      item 19993                                           |
-      item 19994                                           |
-      item 19995                                           |
-      item 19996                                           |
-      item 19997                                           |
-      item 19998                                           |
-      item 19999                                           |
-      item 20000^                                           |
-      -- INSERT (paste) --                                 |
-    ]])
-    feed('<Paste>')
-    screen:expect([[
-      item 19988                                           |
-      item 19989                                           |
-      item 19990                                           |
-      item 19991                                           |
-      item 19992                                           |
-      item 19993                                           |
-      item 19994                                           |
-      item 19995                                           |
-      item 19996                                           |
-      item 19997                                           |
-      item 19998                                           |
-      item 19999                                           |
-      item 20000^                                           |
-      -- INSERT --                       20000,11      Bot |
-    ]])
+  it('support meta + multibyte char mapping', function()
+    add_mapping('<m-ä>', '<m-ä>')
+    check_mapping('<m-ä>', '<m-ä>')
   end)
 end)
 
