@@ -15,7 +15,7 @@ local dedent = global_helpers.dedent
 local neq = global_helpers.neq
 local map = global_helpers.map
 local eq = global_helpers.eq
-local ok = global_helpers.ok
+local trim = global_helpers.trim
 
 -- C constants.
 local NULL = ffi.cast('void*', 0)
@@ -120,10 +120,6 @@ local deinit = only_separate(function()
   end
 end)
 
-local function trim(s)
-  return s:match('^%s*(.*%S)') or ''
-end
-
 -- a Set that keeps around the lines we've already seen
 local cdefs_init = Set:new()
 local cdefs_mod = nil
@@ -460,8 +456,8 @@ else
         if bytes_written == -1 then
           local err = ffi.errno(0)
           if err ~= ffi.C.kPOSIXErrnoEINTR then
-            assert(false, ("write() error: %u: %s"):format(
-                err, ffi.string(ffi.C.strerror(err))))
+            assert(false, ("write() error: %u: %s ('%s')"):format(
+                err, ffi.string(ffi.C.strerror(err)), s))
           end
         elseif bytes_written == 0 then
           break
@@ -645,16 +641,16 @@ local function itp_child(wr, func)
     s = s:sub(1, hook_msglen - 2)
     sc.write(wr, '>' .. s .. (' '):rep(hook_msglen - 2 - #s) .. '\n')
   end
-  local err, emsg = pcall(init)
-  if err then
+  local status, result = pcall(init)
+  if status then
     collectgarbage('stop')
     child_sethook(wr)
-    err, emsg = pcall(func)
+    status, result = pcall(func)
     debug.sethook()
   end
-  emsg = tostring(emsg)
   sc.write(wr, trace_end_msg)
-  if not err then
+  if not status then
+    local emsg = tostring(result)
     if #emsg > 99999 then
       emsg = emsg:sub(1, 99999)
     end
@@ -668,7 +664,7 @@ local function itp_child(wr, func)
   collectgarbage()
   sc.write(wr, '$\n')
   sc.close(wr)
-  sc.exit(err and 0 or 1)
+  sc.exit(status and 0 or 1)
 end
 
 local function check_child_err(rd)
@@ -842,9 +838,6 @@ local module = {
   cimport = cimport,
   cppimport = cppimport,
   internalize = internalize,
-  ok = ok,
-  eq = eq,
-  neq = neq,
   ffi = ffi,
   lib = lib,
   cstr = cstr,
@@ -869,6 +862,7 @@ local module = {
   ptr2key = ptr2key,
   debug_log = debug_log,
 }
+module = global_helpers.tbl_extend('error', module, global_helpers)
 return function()
   return module
 end
