@@ -261,7 +261,7 @@ describe('path.c', function()
   end)
 end)
 
-describe('path_shorten_fname_if_possible', function()
+describe('path_try_shorten_fname', function()
   local cwd = lfs.currentdir()
 
   before_each(function()
@@ -273,22 +273,22 @@ describe('path_shorten_fname_if_possible', function()
     lfs.rmdir('ut_directory')
   end)
 
-  describe('path_shorten_fname_if_possible', function()
+  describe('path_try_shorten_fname', function()
     itp('returns shortened path if possible', function()
       lfs.chdir('ut_directory')
       local full = to_cstr(lfs.currentdir() .. '/subdir/file.txt')
-      eq('subdir/file.txt', (ffi.string(cimp.path_shorten_fname_if_possible(full))))
+      eq('subdir/file.txt', (ffi.string(cimp.path_try_shorten_fname(full))))
     end)
 
     itp('returns `full_path` if a shorter version is not possible', function()
       local old = lfs.currentdir()
       lfs.chdir('ut_directory')
       local full = old .. '/subdir/file.txt'
-      eq(full, (ffi.string(cimp.path_shorten_fname_if_possible(to_cstr(full)))))
+      eq(full, (ffi.string(cimp.path_try_shorten_fname(to_cstr(full)))))
     end)
 
     itp('returns NULL if `full_path` is NULL', function()
-      eq(NULL, (cimp.path_shorten_fname_if_possible(NULL)))
+      eq(NULL, (cimp.path_try_shorten_fname(NULL)))
     end)
   end)
 end)
@@ -585,22 +585,22 @@ describe('path.c', function()
     end)
   end)
 
-  describe('path_is_absolute_path', function()
-    local function path_is_absolute_path(filename)
+  describe('path_is_absolute', function()
+    local function path_is_absolute(filename)
       filename = to_cstr(filename)
-      return cimp.path_is_absolute_path(filename)
+      return cimp.path_is_absolute(filename)
     end
 
     itp('returns true if filename starts with a slash', function()
-      eq(OK, path_is_absolute_path('/some/directory/'))
+      eq(OK, path_is_absolute('/some/directory/'))
     end)
 
     itp('returns true if filename starts with a tilde', function()
-      eq(OK, path_is_absolute_path('~/in/my/home~/directory'))
+      eq(OK, path_is_absolute('~/in/my/home~/directory'))
     end)
 
     itp('returns false if filename starts not with slash nor tilde', function()
-      eq(FAIL, path_is_absolute_path('not/in/my/home~/directory'))
+      eq(FAIL, path_is_absolute('not/in/my/home~/directory'))
     end)
   end)
 end)
